@@ -2,7 +2,7 @@
  * ProGuard -- shrinking, optimization, obfuscation, and preverification
  *             of Java bytecode.
  *
- * Copyright (c) 2002-2008 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2009 Eric Lafortune (eric@graphics.cornell.edu)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -54,24 +54,23 @@ public class InputReader
     public void execute(ClassPool programClassPool,
                         ClassPool libraryClassPool) throws IOException
     {
-        WarningPrinter warningPrinter = configuration.warn ?
-            new WarningPrinter(System.err) :
-            null;
-
-        WarningPrinter notePrinter = configuration.note ?
-            new WarningPrinter(System.out) :
-            null;
-
-        DuplicateClassPrinter duplicateClassPrinter = configuration.note ?
-            new DuplicateClassPrinter(notePrinter) :
-            null;
-
         // Check if we have at least some input classes.
         if (configuration.programJars == null)
         {
             throw new IOException("The input is empty. You have to specify one or more '-injars' options");
         }
 
+        // Perform some sanity checks on the class paths.
+        checkInputOutput(configuration.libraryJars,
+                         configuration.programJars);
+        checkInputOutput(configuration.programJars,
+                         configuration.programJars);
+
+        WarningPrinter warningPrinter = new WarningPrinter(System.err, configuration.warn);
+        WarningPrinter notePrinter    = new WarningPrinter(System.out, configuration.note);
+
+        DuplicateClassPrinter duplicateClassPrinter = new DuplicateClassPrinter(notePrinter);
+
         // Read the program class files.
         // Prepare a data entry reader to filter all classes,
         // which are then decoded to classes by a class reader,
@@ -111,32 +110,58 @@ public class InputReader
         }
 
         // Print out a summary of the notes, if necessary.
-        if (notePrinter != null)
+        int noteCount = notePrinter.getWarningCount();
+        if (noteCount > 0)
         {
-            int noteCount = notePrinter.getWarningCount();
-            if (noteCount > 0)
-            {
-                System.err.println("Note: there were " + noteCount +
-                                   " duplicate class definitions.");
-            }
+            System.err.println("Note: there were " + noteCount +
+                               " duplicate class definitions.");
         }
 
         // Print out a summary of the warnings, if necessary.
-        if (warningPrinter != null)
+        int warningCount = warningPrinter.getWarningCount();
+        if (warningCount > 0)
         {
-            int warningCount = warningPrinter.getWarningCount();
-            if (warningCount > 0)
-            {
-                System.err.println("Warning: there were " + warningCount +
-                                   " classes in incorrectly named files.");
-                System.err.println("         You should make sure all file names correspond to their class names.");
-                System.err.println("         The directory hierarchies must correspond to the package hierarchies.");
+            System.err.println("Warning: there were " + warningCount +
+                               " classes in incorrectly named files.");
+            System.err.println("         You should make sure all file names correspond to their class names.");
+            System.err.println("         The directory hierarchies must correspond to the package hierarchies.");
 
-                if (!configuration.ignoreWarnings)
+            if (!configuration.ignoreWarnings)
+            {
+                System.err.println("         If you don't mind the mentioned classes not being written out,");
+                System.err.println("         you could try your luck using the '-ignorewarnings' option.");
+                throw new IOException("Please correct the above warnings first.");
+            }
+        }
+    }
+
+
+    /**
+     * Performs some sanity checks on the class paths.
+     */
+    private void checkInputOutput(ClassPath inputClassPath,
+                                  ClassPath outputClassPath)
+    throws IOException
+    {
+        if (inputClassPath == null ||
+            outputClassPath == null)
+        {
+            return;
+        }
+
+        for (int index1 = 0; index1 < inputClassPath.size(); index1++)
+        {
+            ClassPathEntry entry1 = inputClassPath.get(index1);
+            if (!entry1.isOutput())
+            {
+                for (int index2 = 0; index2 < outputClassPath.size(); index2++)
                 {
-                    System.err.println("         If you don't mind the mentioned classes not being written out,");
-                    System.err.println("         you could try your luck using the '-ignorewarnings' option.");
-                    throw new IOException("Please correct the above warnings first.");
+                    ClassPathEntry entry2 = outputClassPath.get(index2);
+                    if (entry2.isOutput() &&
+                        entry2.getName().equals(entry1.getName()))
+                    {
+                        throw new IOException("Input jars and output jars must be different ["+entry1.getName()+"]");
+                    }
                 }
             }
         }
