@@ -2,7 +2,7 @@
  * ProGuard -- shrinking, optimization, obfuscation, and preverification
  *             of Java bytecode.
  *
- * Copyright (c) 2002-2008 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2009 Eric Lafortune (eric@graphics.cornell.edu)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -37,4 +37,14 @@ public class LineNumberInfo
     public LineNumberInfo()
     {
     }
+
+
+    /**
+     * Creates an initialized LineNumberInfo.
+     */
+    public LineNumberInfo(int u2startPC, int u2lineNumber)
+    {
+        this.u2startPC    = u2startPC;
+        this.u2lineNumber = u2lineNumber;
+    }
 }
