@@ -2,7 +2,7 @@
  * ProGuard -- shrinking, optimization, obfuscation, and preverification
  *             of Java bytecode.
  *
- * Copyright (c) 2002-2008 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2009 Eric Lafortune (eric@graphics.cornell.edu)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -45,6 +45,31 @@ public class LocalVariableInfo
     public Clazz referencedClass;
 
 
+    /**
+     * Creates an uninitialized LocalVariableInfo.
+     */
+    public LocalVariableInfo()
+    {
+    }
+
+
+    /**
+     * Creates an initialized LocalVariableInfo.
+     */
+    public LocalVariableInfo(int   u2startPC,
+                             int   u2length,
+                             int   u2nameIndex,
+                             int   u2descriptorIndex,
+                             int   u2index)
+    {
+        this.u2startPC         = u2startPC;
+        this.u2length          = u2length;
+        this.u2nameIndex       = u2nameIndex;
+        this.u2descriptorIndex = u2descriptorIndex;
+        this.u2index           = u2index;
+    }
+
+
     /**
      * Lets the referenced class accept the given visitor.
      */
