@@ -2,7 +2,7 @@
  * ProGuard -- shrinking, optimization, obfuscation, and preverification
  *             of Java bytecode.
  *
- * Copyright (c) 2002-2007 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2009 Eric Lafortune (eric@graphics.cornell.edu)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -62,8 +62,10 @@ final class ClassSpecificationDialog extends JDialog
 
     private final JRadioButton[] publicRadioButtons;
     private final JRadioButton[] finalRadioButtons;
-    private final JRadioButton[] interfaceRadioButtons;
     private final JRadioButton[] abstractRadioButtons;
+    private final JRadioButton[] enumRadioButtons;
+    private final JRadioButton[] annotationRadioButtons;
+    private final JRadioButton[] interfaceRadioButtons;
 
     private final JTextField annotationTypeTextField        = new JTextField(20);
     private final JTextField classNameTextField             = new JTextField(20);
@@ -77,7 +79,7 @@ final class ClassSpecificationDialog extends JDialog
 
     public ClassSpecificationDialog(JFrame owner, boolean fullKeepOptions)
     {
-        super(owner, true);
+        super(owner, msg("specifyClasses"), true);
         setResizable(true);
 
         // Create some constraints that can be reused.
@@ -194,10 +196,12 @@ final class ClassSpecificationDialog extends JDialog
         accessPanel.add(tip(new JLabel(msg("dontCare")), "dontCareTip"), labelConstraints);
         accessPanel.add(Box.createGlue(),                                constraintsLastStretch);
 
-        publicRadioButtons    = addRadioButtonTriplet("Public",    accessPanel);
-        finalRadioButtons     = addRadioButtonTriplet("Final",     accessPanel);
-        interfaceRadioButtons = addRadioButtonTriplet("Interface", accessPanel);
-        abstractRadioButtons  = addRadioButtonTriplet("Abstract",  accessPanel);
+        publicRadioButtons     = addRadioButtonTriplet("Public",     accessPanel);
+        finalRadioButtons      = addRadioButtonTriplet("Final",      accessPanel);
+        abstractRadioButtons   = addRadioButtonTriplet("Abstract",   accessPanel);
+        enumRadioButtons       = addRadioButtonTriplet("Enum",       accessPanel);
+        annotationRadioButtons = addRadioButtonTriplet("Annotation", accessPanel);
+        interfaceRadioButtons  = addRadioButtonTriplet("Interface",  accessPanel);
 
         // Create the annotation type panel.
         final JPanel annotationTypePanel = new JPanel(layout);
@@ -342,15 +346,15 @@ final class ClassSpecificationDialog extends JDialog
 
 
     /**
-     * Sets the KeepSpecification to be represented in this dialog.
+     * Sets the KeepClassSpecification to be represented in this dialog.
      */
-    public void setKeepSpecification(KeepSpecification keepSpecification)
+    public void setKeepSpecification(KeepClassSpecification keepClassSpecification)
     {
-        boolean markClasses       = keepSpecification.markClasses;
-        boolean markConditionally = keepSpecification.markConditionally;
-        boolean allowShrinking    = keepSpecification.allowShrinking;
-        boolean allowOptimization = keepSpecification.allowOptimization;
-        boolean allowObfuscation  = keepSpecification.allowObfuscation;
+        boolean markClasses       = keepClassSpecification.markClasses;
+        boolean markConditionally = keepClassSpecification.markConditionally;
+        boolean allowShrinking    = keepClassSpecification.allowShrinking;
+        boolean allowOptimization = keepClassSpecification.allowOptimization;
+        boolean allowObfuscation  = keepClassSpecification.allowObfuscation;
 
         // Figure out the proper keep radio button and set it.
         JRadioButton keepOptionRadioButton =
@@ -365,7 +369,7 @@ final class ClassSpecificationDialog extends JDialog
         allowOptimizationCheckBox.setSelected(allowOptimization);
         allowObfuscationCheckBox .setSelected(allowObfuscation);
 
-        setClassSpecification(keepSpecification);
+        setClassSpecification(keepClassSpecification);
     }
 
 
@@ -386,10 +390,12 @@ final class ClassSpecificationDialog extends JDialog
         commentsTextArea.setText(comments == null ? "" : comments);
 
         // Set the access radio buttons.
-        setClassSpecificationRadioButtons(classSpecification, ClassConstants.INTERNAL_ACC_PUBLIC,    publicRadioButtons);
-        setClassSpecificationRadioButtons(classSpecification, ClassConstants.INTERNAL_ACC_FINAL,     finalRadioButtons);
-        setClassSpecificationRadioButtons(classSpecification, ClassConstants.INTERNAL_ACC_INTERFACE, interfaceRadioButtons);
-        setClassSpecificationRadioButtons(classSpecification, ClassConstants.INTERNAL_ACC_ABSTRACT,  abstractRadioButtons);
+        setClassSpecificationRadioButtons(classSpecification, ClassConstants.INTERNAL_ACC_PUBLIC,      publicRadioButtons);
+        setClassSpecificationRadioButtons(classSpecification, ClassConstants.INTERNAL_ACC_FINAL,       finalRadioButtons);
+        setClassSpecificationRadioButtons(classSpecification, ClassConstants.INTERNAL_ACC_ABSTRACT,    abstractRadioButtons);
+        setClassSpecificationRadioButtons(classSpecification, ClassConstants.INTERNAL_ACC_ENUM,        enumRadioButtons);
+        setClassSpecificationRadioButtons(classSpecification, ClassConstants.INTERNAL_ACC_ANNOTATTION, annotationRadioButtons);
+        setClassSpecificationRadioButtons(classSpecification, ClassConstants.INTERNAL_ACC_INTERFACE,   interfaceRadioButtons);
 
         // Set the class and annotation text fields.
         annotationTypeTextField       .setText(annotationType        == null ? ""  : ClassUtil.externalType(annotationType));
@@ -403,9 +409,9 @@ final class ClassSpecificationDialog extends JDialog
 
 
     /**
-     * Returns the KeepSpecification currently represented in this dialog.
+     * Returns the KeepClassSpecification currently represented in this dialog.
      */
-    public KeepSpecification getKeepSpecification()
+    public KeepClassSpecification getKeepSpecification()
     {
         boolean markClasses       = !keepClassMembersRadioButton     .isSelected();
         boolean markConditionally = keepClassesWithMembersRadioButton.isSelected();
@@ -413,7 +419,7 @@ final class ClassSpecificationDialog extends JDialog
         boolean allowOptimization = allowOptimizationCheckBox        .isSelected();
         boolean allowObfuscation  = allowObfuscationCheckBox         .isSelected();
 
-        return new KeepSpecification(markClasses,
+        return new KeepClassSpecification(markClasses,
                                      markConditionally,
                                      allowShrinking,
                                      allowOptimization,
@@ -444,10 +450,12 @@ final class ClassSpecificationDialog extends JDialog
                                    extendsClassName.equals("")      ? null : ClassUtil.internalClassName(extendsClassName));
 
         // Also get the access radio button settings.
-        getClassSpecificationRadioButtons(classSpecification, ClassConstants.INTERNAL_ACC_PUBLIC,    publicRadioButtons);
-        getClassSpecificationRadioButtons(classSpecification, ClassConstants.INTERNAL_ACC_FINAL,     finalRadioButtons);
-        getClassSpecificationRadioButtons(classSpecification, ClassConstants.INTERNAL_ACC_INTERFACE, interfaceRadioButtons);
-        getClassSpecificationRadioButtons(classSpecification, ClassConstants.INTERNAL_ACC_ABSTRACT,  abstractRadioButtons);
+        getClassSpecificationRadioButtons(classSpecification, ClassConstants.INTERNAL_ACC_PUBLIC,      publicRadioButtons);
+        getClassSpecificationRadioButtons(classSpecification, ClassConstants.INTERNAL_ACC_FINAL,       finalRadioButtons);
+        getClassSpecificationRadioButtons(classSpecification, ClassConstants.INTERNAL_ACC_ABSTRACT,    abstractRadioButtons);
+        getClassSpecificationRadioButtons(classSpecification, ClassConstants.INTERNAL_ACC_ENUM,        enumRadioButtons);
+        getClassSpecificationRadioButtons(classSpecification, ClassConstants.INTERNAL_ACC_ANNOTATTION, annotationRadioButtons);
+        getClassSpecificationRadioButtons(classSpecification, ClassConstants.INTERNAL_ACC_INTERFACE,   interfaceRadioButtons);
 
         // Get the keep class member option lists.
         classSpecification.fieldSpecifications  = memberSpecificationsPanel.getMemberSpecifications(true);
@@ -482,8 +490,8 @@ final class ClassSpecificationDialog extends JDialog
      * flags of the given keep option.
      */
     private void setClassSpecificationRadioButtons(ClassSpecification classSpecification,
-                                                    int                 flag,
-                                                    JRadioButton[]      radioButtons)
+                                                   int                flag,
+                                                   JRadioButton[]     radioButtons)
     {
         int index = (classSpecification.requiredSetAccessFlags   & flag) != 0 ? 0 :
                     (classSpecification.requiredUnsetAccessFlags & flag) != 0 ? 1 :
@@ -497,8 +505,8 @@ final class ClassSpecificationDialog extends JDialog
      * button triplet.
      */
     private void getClassSpecificationRadioButtons(ClassSpecification classSpecification,
-                                                    int                 flag,
-                                                    JRadioButton[]      radioButtons)
+                                                   int                flag,
+                                                   JRadioButton[]     radioButtons)
     {
         if      (radioButtons[0].isSelected())
         {
