@@ -2,7 +2,7 @@
  * ProGuard -- shrinking, optimization, obfuscation, and preverification
  *             of Java bytecode.
  *
- * Copyright (c) 2002-2008 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2009 Eric Lafortune (eric@graphics.cornell.edu)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -52,17 +52,24 @@ public class DataEntryCopier implements DataEntryReader
     {
         try
         {
-            // Get the output entry corresponding to this input entry.
-            OutputStream outputStream = dataEntryWriter.getOutputStream(dataEntry);
-            if (outputStream != null)
+            if (dataEntry.isDirectory())
             {
-                InputStream inputStream = dataEntry.getInputStream();
+                dataEntryWriter.createDirectory(dataEntry);
+            }
+            else
+            {
+                // Get the output entry corresponding to this input entry.
+                OutputStream outputStream = dataEntryWriter.getOutputStream(dataEntry);
+                if (outputStream != null)
+                {
+                    InputStream inputStream = dataEntry.getInputStream();
 
-                // Copy the data from the input entry to the output entry.
-                copyData(inputStream, outputStream);
+                    // Copy the data from the input entry to the output entry.
+                    copyData(inputStream, outputStream);
 
-                // Close the data entries.
-                dataEntry.closeInputStream();
+                    // Close the data entries.
+                    dataEntry.closeInputStream();
+                }
             }
         }
         catch (IOException ex)
@@ -72,14 +79,12 @@ public class DataEntryCopier implements DataEntryReader
     }
 
 
-    // Small utility methods.
-
     /**
      * Copies all data that it can read from the given input stream to the
      * given output stream.
      */
-    private void copyData(InputStream  inputStream,
-                          OutputStream outputStream)
+    protected void copyData(InputStream  inputStream,
+                            OutputStream outputStream)
     throws IOException
     {
         while (true)
