@@ -1,8 +1,8 @@
-/* $Id: ZipDataEntry.java,v 1.4.2.2 2007/01/18 21:31:52 eric Exp $
+/*
+ * ProGuard -- shrinking, optimization, obfuscation, and preverification
+ *             of Java bytecode.
  *
- * ProGuard -- shrinking, optimization, and obfuscation of Java class files.
- *
- * Copyright (c) 2002-2007 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2009 Eric Lafortune (eric@graphics.cornell.edu)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -20,11 +20,11 @@
  */
 package proguard.io;
 
+import proguard.classfile.ClassConstants;
+
 import java.io.*;
 import java.util.zip.*;
 
-import proguard.classfile.ClassConstants;
-
 /**
  * This <code>DataEntry</code> represents a ZIP entry.
  *
@@ -32,9 +32,9 @@ import proguard.classfile.ClassConstants;
  */
 public class ZipDataEntry implements DataEntry
 {
-    private DataEntry      parent;
-    private ZipEntry       zipEntry;
-    private ZipInputStream zipInputStream;
+    private final DataEntry      parent;
+    private final ZipEntry       zipEntry;
+    private       ZipInputStream zipInputStream;
 
 
     public ZipDataEntry(DataEntry      parent,
@@ -51,9 +51,22 @@ public class ZipDataEntry implements DataEntry
 
     public String getName()
     {
-        // Chop the directory name from the file name and get the right separators.
-        return zipEntry.getName()
+        // Get the right separators.
+        String name = zipEntry.getName()
             .replace(File.separatorChar, ClassConstants.INTERNAL_PACKAGE_SEPARATOR);
+
+        // Chop the trailing directory slash, if any.
+        int length = name.length();
+        return length > 0 &&
+               name.charAt(length-1) == ClassConstants.INTERNAL_PACKAGE_SEPARATOR ?
+                   name.substring(0, length -1) :
+                   name;
+    }
+
+
+    public boolean isDirectory()
+    {
+        return zipEntry.isDirectory();
     }
 
 
