@@ -2,7 +2,7 @@
  * ProGuard -- shrinking, optimization, obfuscation, and preverification
  *             of Java bytecode.
  *
- * Copyright (c) 2002-2008 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2009 Eric Lafortune (eric@graphics.cornell.edu)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -136,7 +136,8 @@ public class MemberNameConflictFixer implements MemberVisitor
                 warningPrinter != null)
             {
                 descriptor = member.getDescriptor(clazz);
-                warningPrinter.print("Warning: " + ClassUtil.externalClassName(clazz.getName()) +
+                warningPrinter.print(clazz.getName(),
+                                     "Warning: " + ClassUtil.externalClassName(clazz.getName()) +
                                                    (isField ?
                                                        ": field '" + ClassUtil.externalFullFieldDescription(0, name, descriptor) :
                                                        ": method '" + ClassUtil.externalFullMethodDescription(clazz.getName(), 0, name, descriptor)) +
