@@ -2,7 +2,7 @@
  * ProGuard -- shrinking, optimization, obfuscation, and preverification
  *             of Java bytecode.
  *
- * Copyright (c) 2002-2008 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2009 Eric Lafortune (eric@graphics.cornell.edu)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -41,8 +41,8 @@ implements   InstructionVisitor,
              MemberVisitor
 {
     // Parameters for the visitor methods.
-    private boolean reading;
-    private boolean writing;
+    private boolean reading = true;
+    private boolean writing = true;
 
 
     // Implementations for InstructionVisitor.
