@@ -2,7 +2,7 @@
  * ProGuard -- shrinking, optimization, obfuscation, and preverification
  *             of Java bytecode.
  *
- * Copyright (c) 2002-2008 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2009 Eric Lafortune (eric@graphics.cornell.edu)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -101,7 +101,7 @@ implements   AttributeVisitor
             // Delete unused local variables from the local variable frame.
             variableEditor.reset(maxLocals);
 
-            for (int variableIndex = parameterSize+1; variableIndex < maxLocals; variableIndex++)
+            for (int variableIndex = parameterSize; variableIndex < maxLocals; variableIndex++)
             {
                 // Is the variable not required?
                 if (!variableUsageMarker.isVariableUsed(variableIndex))
